"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.buildImportMap = buildImportMap;
exports.rewriteModuleSpecifier = rewriteModuleSpecifier;
var _parsePackageName = require("./parsePackageName");
var _config = require("./config");
function buildImportMap(workspacePath, {
  externalDependencies,
  externalResolutions,
  selectiveCDNResolutions
}) {
  const externalCdnTemplate = (0, _config.getConfig)('externalCdnTemplate', workspacePath);

  // Add react-dom if only react is specified in the dependencies
  if (!externalDependencies['react-dom']) {
    externalDependencies['react-dom'] = externalDependencies['react'];
  }
  if (!externalResolutions['react-dom']) {
    externalResolutions['react-dom'] = externalResolutions['react'];
  }

  // Generate an import map
  const importMap = new Map(Object.entries(externalDependencies).map(([name, version]) => {
    if (!externalResolutions[name]) {
      throw new Error(`Dependency ${name} found in package.json but not in lockfile. Have you installed your dependencies?`);
    }
    return [name, externalCdnTemplate.replace('[name]', name).replace('[version]', version ?? externalResolutions[name]).replace('[resolution]', externalResolutions[name]).replace('[selectiveCDNResolutions]', selectiveCDNResolutions ? Object.entries(selectiveCDNResolutions).map(([key, value]) => `${key}@${value}`).join(',') : '')];
  }));
  return importMap;
}
function rewriteModuleSpecifier(importMap, moduleSpecifier) {
  const {
    dependencyName,
    submodule
  } = (0, _parsePackageName.parsePackageName)(moduleSpecifier);
  // Find dependency name (no submodule) in the pre-built import map
  const dependencyUrl = dependencyName ? importMap.get(dependencyName) : undefined;
  if (dependencyUrl) {
    // Rewrite the path taking the submodule into account
    return `${dependencyUrl}${submodule ? `/${submodule}` : ''}`;
  }
}
//# sourceMappingURL=buildImportMap.js.map