"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.getConfig = getConfig;
var _cosmiconfig = require("cosmiconfig");
var _path = _interopRequireDefault(require("path"));
function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }
// Where cosmiconfig can look for the configuration
const searchPlaces = ['.modular.js', 'package.json', `.modularrc`, `.modularrc.json`, `.modularrc.yaml`, `.modularrc.yml`, `.modularrc.js`, `.modularrc.cjs`, `modular.config.js`, `modular.config.cjs`];

/**
 * Configuration file interface
 */

/**
 * Defaults and env variable overrides
 */
const defs = {
  useModularEsbuild: {
    default: false,
    override: process.env.USE_MODULAR_ESBUILD === 'true' || process.env.USE_MODULAR_ESBUILD === 'false' ? process.env.USE_MODULAR_ESBUILD === 'true' : undefined
  },
  externalCdnTemplate: {
    default: 'https://esm.sh/[name]@[version]',
    override: process.env.EXTERNAL_CDN_TEMPLATE
  },
  externalBlockList: {
    default: [],
    override: process.env.EXTERNAL_BLOCK_LIST ? process.env.EXTERNAL_BLOCK_LIST.split(',') : undefined
  },
  externalAllowList: {
    default: ['**'],
    override: process.env.EXTERNAL_ALLOW_LIST ? process.env.EXTERNAL_ALLOW_LIST.split(',') : undefined
  },
  publicUrl: {
    default: '',
    override: process.env.PUBLIC_URL
  },
  generateSourceMap: {
    default: true,
    override: process.env.GENERATE_SOURCEMAP === 'true' || process.env.GENERATE_SOURCEMAP === 'false' ? process.env.GENERATE_SOURCEMAP === 'true' : undefined
  }
};
const explorer = (0, _cosmiconfig.cosmiconfigSync)('modular', {
  searchPlaces
});
const configurations = new Map();

/**
 * Searches for configuration files in a given workspace,
 * first checks if there's an already loaded & cached configuration, otherwise
 * runs a cosmiconfig search and saves it for future use
 * @param workspacePath Location to search
 * @returns cosmiconfig configuration, if any found
 */
function searchConfig(workspacePath) {
  const cachedConfig = configurations.get(workspacePath);
  if (cachedConfig !== undefined) {
    return cachedConfig;
  } else {
    const config = explorer.search(_path.default.join(workspacePath, 'package.json'));
    configurations.set(workspacePath, config);
    return config;
  }
}

/**
 * Get the configured value in a workspace for a given configuration field.
 * @param field Field containing the configuration variable to read
 * @param workspacePath Path of workspace to which configuration applies
 * @returns configured value:
 * - the override environment variable if configured
 * - the value stated in the config file if provided
 * - the default value if neither environment variable nor the config file are provided
 */
function getConfig(field, workspacePath) {
  const configResult = searchConfig(workspacePath);
  const configValue = configResult ? configResult.config[field] : undefined;
  return defs[field].override ?? configValue ?? defs[field].default;
}
//# sourceMappingURL=config.js.map